<?php

/**
 * 
 * Rollout plugin - ssh
 * 
 * Run a SSH command on remote targets
 *
 * @author <axel.hahn@iml.unibe.ch>
 * 
 * 2024-09-02  Axel   php8 only; added variable types; short array syntax
 */
class rollout_ssh extends rollout_base {

    /**
     * Get an array with shell commands to check requirements if the plugin
     * can work
     * 
     * @return array
     */
    public function checkRequirements(): array 
    {
        // no specific checks needed ... always empty
        return [];
    }

    /**
     * Get an array with shell commands to check access to a deploy target
     * 
     * @return array
     */
    public function checkConnectionToTarget(): array
    {
        // do nothing ... always empty
        return [];
    }

    /**
     * Get an array with shell commands to execute for deployment of built file
     * 
     * @param  string   $sPhase
     * @param  boolean  $bMask   Flag for public output; if true then mask your secrets
     * @return array
     */
    public function getDeployCommands(string $sPhase, bool $bMask = false): array
    {
        $aReturn=[];
        $aConfig=$this->getConfig($sPhase);
        
        // loop over hosts and create shell commands for it
        foreach(explode(',', $aConfig['hosts']) as $sMyHost){
            $aReturn[]= sprintf($aConfig['addkeycommand'], $sMyHost, $sMyHost);
            $aReturn[]='ssh '
                    . ($aConfig['privatekey'] ? '-i \''.$aConfig['privatekey'].'\' ' : '')
                    . ($aConfig['user'] ? $aConfig['user'].'@' : '')
                    . $sMyHost.' '
                    . '\''.$aConfig['command'].'\''
                    ;
        }
        
        return $aReturn;
    }
    
}
